package PRFramework.Core.SupervisedClassifiers.EmergingPatterns.Classifiers;

import PRFramework.Core.Common.Instance;
import PRFramework.Core.Common.InstanceModel;
import PRFramework.Core.Common.NominalFeature;
import PRFramework.Core.SupervisedClassifiers.EmergingPatterns.IEmergingPattern;
import PRFramework.Core.SupervisedClassifiers.EmergingPatterns.Miners.*;

import java.awt.dnd.InvalidDnDOperationException;
import java.io.*;
import java.util.*;

public class PBC4occ implements  Serializable{

    private  Collection<IEmergingPattern> EmergingPatterns;

    private double averageScore;
    private int _numPatterns;
    private int negativeClassIndex;
    public double[] Classify(Instance instance)
    {


        double result = getScore(instance);
        double[] probabilities = new double[2];
        probabilities[negativeClassIndex] = Math.exp(-0.5 * Math.pow(averageScore / result, 2));
        probabilities[1 - negativeClassIndex] = 1 - probabilities[negativeClassIndex];
        return probabilities;
    }

    private OneClassRandomForestMiner PatternsMiner = null;

    public void Train(InstanceModel model, Collection<Instance> instances, NominalFeature classFeature)
    {
        String negativeClassLabel = PatternsMiner.getNegativeClassLabel();
        String[] classValues = model.getClassFeature().Values;
        if (classValues[0].equals(negativeClassLabel)){
            negativeClassIndex = 0;
        } else if (classValues[1].equals(negativeClassLabel)) {
            negativeClassIndex = 1;
        } else {
            throw new IllegalStateException("Negative class label not found.");
        }
        System.out.println(negativeClassIndex);

        ArrayList<Instance> trainingDataset = new ArrayList<>(instances);
        Collection<IEmergingPattern>  patterns = PatternsMiner.mine(model, trainingDataset, classFeature);


        //treeCount = PatternsMiner.getTreeCount();
//        HashSet<String> patternStrSet = new HashSet<>();
//        List<IEmergingPattern> filteredPatterns = new ArrayList<>();
//
//        for (IEmergingPattern pattern : patterns)
//        {
//            String currentStr = pattern.toString();
//            if (patternStrSet.add(currentStr))
//                filteredPatterns.add(pattern);
//        }

        Train(model, trainingDataset, new ArrayList<>(patterns), classFeature);
    }

    public void Train(InstanceModel model, List<Instance> trainingDataset,
                      List<IEmergingPattern> patterns, NominalFeature classFeature)
    {
        //List<Instance> trainingDataset = new ArrayList<>(instances);

        for (int i = 0; i < patterns.size(); i++) {

            patterns.get(i).UpdateCountsAndSupport(trainingDataset);
        }
        //patterns.sort(Comparator.comparingDouble(pattern -> - Arrays.stream(pattern.getSupports()).max().getAsDouble()));

        EmergingPatterns = patterns;
        //SerializePatterns();
        _countSum = 0;

        for (IEmergingPattern pattern : patterns)
        {
            _countSum += pattern.getCounts()[0] + pattern.getCounts()[1];
        }

        _isTrained = true;
        _numPatterns= EmergingPatterns.size();
        double scoreSum = 0;
        for (Instance instance :
                trainingDataset) {
            scoreSum += getScore(instance);
        }
        averageScore = scoreSum / trainingDataset.size();

    }

    public double getScore(Instance instance)
    {
        if (!_isTrained)
            throw new InvalidDnDOperationException("Unable to classify with {nameof(PBC4cip)}: Untrained classifier!");

        double result = 0;
        for (IEmergingPattern emergingPattern: EmergingPatterns)
            if (emergingPattern.isMatch(instance))
            {
                result += emergingPattern.getCounts()[0] + emergingPattern.getCounts()[1];
            }
        return 1.0 * result / _countSum; // countSum = numTrees * numInstances
    }

    private boolean _isTrained = false;

    private double _countSum = 0;

    public Collection<IEmergingPattern> getEmergingPatterns() {
        return EmergingPatterns;
    }

    public void setEmergingPatterns(Collection<IEmergingPattern> emergingPatterns) {
        for (IEmergingPattern pattern:
             EmergingPatterns) {
            pattern = null;

        }
        EmergingPatterns = emergingPatterns;
    }

    public OneClassRandomForestMiner getPatternsMiner() {
        return PatternsMiner;
    }

    public void setPatternsMiner(OneClassRandomForestMiner patternsMiner) {
        PatternsMiner = patternsMiner;
    }

    public String getTopPatterns()
    {
        List<IEmergingPattern> patterns = new ArrayList<>(EmergingPatterns);
        //patterns.sort(Comparator.comparingInt(o -> - o.getItems().size()));
        patterns.sort(Comparator.comparingDouble(pattern -> Arrays.stream(pattern.getSupports()).max().getAsDouble()));

        StringBuilder strBuilder = new StringBuilder("");
        for (IEmergingPattern pattern : patterns)
        {
            strBuilder.append(pattern);
            strBuilder.append("\n");
        }
        return strBuilder.toString();
    }
    public void SerializePatterns(){
        try
        {
            FileOutputStream fos = new FileOutputStream("/home/leo/Desktop/patterns.txt");
            ObjectOutputStream oos = new ObjectOutputStream(fos);
            oos.writeObject(EmergingPatterns);
            oos.close();
            fos.close();
        }
        catch (IOException ioe)
        {
            ioe.printStackTrace();
        }
    }

    public int getNumPatterns() {
        return _numPatterns;
    }
}



