package PRFramework.Core.SupervisedClassifiers.EmergingPatterns.Classifiers;

import PRFramework.Core.Common.Instance;
import PRFramework.Core.Common.InstanceModel;
import PRFramework.Core.Common.NominalFeature;
import PRFramework.Core.SupervisedClassifiers.EmergingPatterns.IEmergingPattern;
import PRFramework.Core.SupervisedClassifiers.EmergingPatterns.Miners.TreeBasedMiner;
import PRFramework.Core.SupervisedClassifiers.EmergingPatterns.Miners.TreeBasedMinerWithoutFiltering;

import java.awt.dnd.InvalidDnDOperationException;
import java.io.Serializable;
import java.util.*;

public class PBC4occ2 implements Serializable {
    private  Collection<IEmergingPattern> EmergingPatterns;
    private int treeCount;
    private double averageScore;
    public double[] Classify(Instance instance)
    {
        if (!_isTrained)
            throw new InvalidDnDOperationException("Unable to classify with {nameof(PBC4cip)}: Untrained classifier!");

        double result = 0;
        int matches = 0;
        for (IEmergingPattern emergingPattern: EmergingPatterns) {
            if (emergingPattern.isMatch(instance)) {
                matches += 1;
                result += emergingPattern.getCounts()[0] + emergingPattern.getCounts()[1];
            }
        }
        //result *= treeCount;
        double[] probabilities = new double[2];
        result = 1.0 * result / EmergingPatterns.size();
        //probabilities[1] = 1.0 * result / EmergingPatterns.size();    // _countSum = numTrees * datasetSize
        probabilities[1] = Math.pow(2, -averageScore / result);
        //probabilities[1] /= averageScore;
        //probabilities[1] = Math.exp(-0.5 * Math.pow((result/_countSum) / averageScore, 2));
        probabilities[0] = 1.0 - probabilities[1];
        return probabilities;
    }

    private TreeBasedMinerWithoutFiltering PatternsMiner = null;

    public void Train(InstanceModel model, Collection<Instance> instances, NominalFeature classFeature)
    {
        List<Instance> trainingDataset = new ArrayList<>(instances);
        Collection<IEmergingPattern>  patterns = PatternsMiner.mine(model, trainingDataset, classFeature);
        treeCount = PatternsMiner.getTreeCount();
        HashSet<String> patternStrSet = new HashSet<>();
        List<IEmergingPattern> filteredPatterns = new ArrayList<>();
        for (IEmergingPattern pattern : patterns)
        {
            String currentStr = pattern.toString();
            //if (patternStrSet.add(currentStr))
                filteredPatterns.add(pattern);
        }

        Train(model, trainingDataset, filteredPatterns, classFeature);
    }

    public void Train(InstanceModel model, Collection<Instance> instances,
                      List<IEmergingPattern> patterns, NominalFeature classFeature)
    {
        List<Instance> trainingDataset = new ArrayList<>(instances);

        for (IEmergingPattern pattern : patterns)
        {
            pattern.UpdateCountsAndSupport(trainingDataset);
        }
        patterns.sort(Comparator.comparingDouble(pattern -> - Arrays.stream(pattern.getSupports()).max().getAsDouble()));

        EmergingPatterns = patterns;
        _countSum = 0;

        for (IEmergingPattern pattern : patterns)
        {
            double[] c = pattern.getCounts();
            _countSum += pattern.getCounts()[0] + pattern.getCounts()[1];
            if(pattern.getCounts()[0] + pattern.getCounts()[1] > _maxCount)
                _maxCount = pattern.getCounts()[0] + pattern.getCounts()[1];
        }
        /*
        System.out.println("-----------");
        System.out.println("Instances: "+ instances.size());
        System.out.println("Patterns: " + patterns.size());
        System.out.println("Count sum: " + _countSum);
        System.out.println("Max count: " + _maxCount);
         */

        _isTrained = true;

        double scoreSum = 0;
        for (Instance instance :
                instances) {
            scoreSum += getScore(instance);
        }
        averageScore = scoreSum / instances.size();
    }


    public double getScore(Instance instance)
    {
        if (!_isTrained)
            throw new InvalidDnDOperationException("Unable to classify with {nameof(PBC4cip)}: Untrained classifier!");

        double result = 0;
        for (IEmergingPattern emergingPattern: EmergingPatterns)
            if (emergingPattern.isMatch(instance))
            {
                result += emergingPattern.getCounts()[0] + emergingPattern.getCounts()[1];
            }
        //System.out.println("Score: " + Double.toString(1.0 * result / treeCount));

        return 1.0 * result / EmergingPatterns.size(); // countSum = numTrees * numInstances
    }

    private double _maxCount = 0;
    private boolean _isTrained = false;

    private double _countSum = 0;

    public Collection<IEmergingPattern> getEmergingPatterns() {
        return EmergingPatterns;
    }

    public void setEmergingPatterns(Collection<IEmergingPattern> emergingPatterns) {
        EmergingPatterns = emergingPatterns;
    }

    public TreeBasedMiner getPatternsMiner() {
        return PatternsMiner;
    }

    public void setPatternsMiner(TreeBasedMinerWithoutFiltering patternsMiner) {
        PatternsMiner = patternsMiner;
    }
}



