package PRFramework.Core.SupervisedClassifiers.EmergingPatterns.Miners;

import PRFramework.Core.Common.Actions.Action;
import PRFramework.Core.Common.Feature;
import PRFramework.Core.Common.Functions.Func2Param;
import PRFramework.Core.Common.Instance;
import PRFramework.Core.Common.InstanceModel;
import PRFramework.Core.Common.NominalFeature;
import PRFramework.Core.Samplers.RandomSampler;
import PRFramework.Core.SupervisedClassifiers.DecisionTrees.Builder.DecisionTreeBuilder;
import PRFramework.Core.SupervisedClassifiers.DecisionTrees.DecisionTree;
import PRFramework.Core.SupervisedClassifiers.DecisionTrees.DecisionTreeClassifier;
import PRFramework.Core.SupervisedClassifiers.EmergingPatterns.EmergingPattern;
import PRFramework.Core.SupervisedClassifiers.EmergingPatterns.EmergingPatternCreator;
import PRFramework.Core.SupervisedClassifiers.EmergingPatterns.PatternTests.AlwaysTrue;
import PRFramework.Core.SupervisedClassifiers.EmergingPatterns.SubsetRelation;
import weka.core.OptionMetadata;

import java.io.IOException;
import java.io.Serializable;
import java.util.*;
import java.util.stream.Collectors;

import static PRFramework.Core.Common.Helpers.ArrayHelper.stream;
import static java.util.stream.Collectors.toList;

public class OneClassBaggingMiner10 extends OneClassRandomForestMiner
{
    private int TreeCount;// { get; set; }
    private int Percent; // { get; set; }
    private int FeatureCount; // { get; set; }

    @Override
    protected void doMine(InstanceModel model, Iterable<Instance> instances, Feature classFeature,
                          EmergingPatternCreator epCreator, Action<EmergingPattern> action)
    {
        _sampler.setRandomGenerator(randNumGen);

        Instance[] instancesArray = stream(instances).toArray(Instance[]::new);

        List<Feature> featuresToConsider = Arrays.stream(model.getFeatures()).filter(f -> Feature.OpInequality(f, classFeature)).collect(toList());

        int featureCount = (FeatureCount != -1) ? FeatureCount : (int)Math.sqrt(featuresToConsider.size());
        //int featureCount = (FeatureCount != -1) ? FeatureCount : (int)Math.Log(featuresToConsider.Count, 2) + 1;

        List<Instance> negativeInstances =
                GetOneClass(Arrays.asList(instancesArray), model, getNegativeClassLabel());
        int percent = (Percent * negativeInstances.size()) / 100;

        setEPTester(new AlwaysTrue());
        HashMap<Instance, Double> backupClassValues = new HashMap<>();
        for (Instance instance : negativeInstances)
        {
            backupClassValues.put(instance, instance.get(classFeature));
        }
        for (int i = 0; i < TreeCount; i++)
        {
            List<Instance> newInstances = Changing2B(negativeInstances, percent, classFeature);
            ((DecisionTreeBuilder)getDecisionTreeBuilder()).OnSelectingFeaturesToConsider =
                    (Func2Param<Collection<Feature>, Integer, Collection<Feature>> & Serializable)(features, level) -> _sampler.SampleWithoutRepetition(featuresToConsider, featureCount);

            DecisionTree tree = getDecisionTreeBuilder().Build(model, newInstances, classFeature);
            //System.out.println("Tree size: " + tree.getLeaves());
            DecisionTreeClassifier treeClassifier = new DecisionTreeClassifier(tree);
            epCreator.ExtractPatterns(treeClassifier, action, classFeature);

            for (Instance instance : negativeInstances)
            {
                instance.set(classFeature, backupClassValues.get(instance));
            }
        }
        System.out.println("Done mining");
    }

    public OneClassBaggingMiner10()
    {
        setNegativeClassLabel("negative");
        Percent = 90;
        TreeCount = 700;
        FeatureCount = -1;
    }

    private List<Instance> Changing2B(List<Instance> dataset, int sampleSize, Feature classFeature) {
        for (int i = 0; i < dataset.size(); i++) {
            int change = randNumGen.nextInt(2);
            Instance instance = dataset.get(i);
            if(change == 1)
                instance.set(classFeature, (int)instance.get(classFeature) == 0 ? 1 : 0);
        }

        return dataset;
    }
    private List<Instance> Changing2(List<Instance> dataset, int sampleSize, Feature classFeature)
    {
        List<Instance> selected = new ArrayList<>();
        HashSet<Integer> selectedIdxs = new HashSet<>();
        for (int i = 0; i < sampleSize; i++)
        {
            int idx = randNumGen.nextInt(dataset.size());
            if (selectedIdxs.add(idx))
            {
                selected.add(dataset.get(idx));
            }
        }

        List<Instance> remaining = new ArrayList<>(dataset);

        for (int i = 0; i < dataset.size(); i++)
            if (!selectedIdxs.contains(i))
            {
                remaining.add(dataset.get(i));
            }

        for (Instance instance : remaining)
            instance.set(classFeature, (int)instance.get(classFeature) == 0 ? 1 : 0);

        List<Instance> result = new ArrayList<>();
        result.addAll(selected);
        result.addAll(remaining);

        return result;
    }
    private List<Instance> GetOneClass(Collection<Instance> instances, InstanceModel model, String classToSelect)
    {
        String className = model.getClassFeature().getName();
        List<Instance> cloned = new ArrayList<>(instances);
        //var classFeature = cloned[0].Model[className];

        NominalFeature classFeature = cloned.get(0).getModel().getClassFeature();
        //var groupedInst = cloned.GroupBy(x => x[classFeature]).ToList();
        Map<Double, List<Instance>> groupedInst = cloned.stream().collect(Collectors.groupingBy(x -> x.get(classFeature)));
        double idxClass = 0;
        String[] classValues = model.getClassFeature().Values;

        for (int i = 0; i < classValues.length; i++)
        {
            String value = classValues[i];
            if (value.toLowerCase().equals(classToSelect.toLowerCase()))
            {
                idxClass = i;
                break;
            }
        }

        for (Double group : groupedInst.keySet()) {
            if (group == idxClass)
                return groupedInst.get(group);
        }
        return null;
    }

    private static RandomSampler _sampler = new RandomSampler();

    @Override
    @OptionMetadata(displayName = "numTrees",
            description = "The number of trees for the random forest.",
            commandLineParamName = "numTrees",
            commandLineParamSynopsis = "-numTrees <string>", displayOrder = 1)
    public int getTreeCount() {
        return TreeCount;
    }

    public void setTreeCount(int treeCount) {
        TreeCount = treeCount;
    }

    @OptionMetadata(displayName = "percent",
            description = "Percentage of something...",
            commandLineParamName = "percent",
            commandLineParamSynopsis = "-percent <string>", displayOrder = 4)
    public int getPercent() {
        return Percent;
    }

    public void setPercent(int percent) {
        Percent = percent;
    }

    @OptionMetadata(displayName = "numFeatures",
            description = "The number of features to consider. If set to -1, the number of features to consider is " +
                    "log_2(features) + 1",
            commandLineParamName = "numFeatures",
            commandLineParamSynopsis = "-numFeatures <string>", displayOrder = 2)
    public int getFeatureCount() {
        return FeatureCount;
    }
    public void setFeatureCount(int featureCount) {
        FeatureCount = featureCount;
    }

}