package weka.classifiers.trees;

import PRFramework.Core.Common.Instance;
import PRFramework.Core.Common.InstanceModel;
import PRFramework.Core.Common.NominalFeature;
import PRFramework.Core.Common.RefObject;
import PRFramework.Core.IO.WekaSerializer;
import PRFramework.Core.SupervisedClassifiers.DecisionTrees.Builder.DecisionTreeBuilder;
import PRFramework.Core.SupervisedClassifiers.DecisionTrees.DistributionEvaluators.Twoing;
import PRFramework.Core.SupervisedClassifiers.DecisionTrees.DistributionTesters.PureNodeStopCondition;
import PRFramework.Core.SupervisedClassifiers.DecisionTrees.SplitIteratorProviders.StandardSplitIteratorProvider;
import PRFramework.Core.SupervisedClassifiers.EmergingPatterns.IEmergingPattern;
import PRFramework.Core.SupervisedClassifiers.EmergingPatterns.Miners.IEmergingPatternMiner;
import PRFramework.Core.SupervisedClassifiers.EmergingPatterns.Miners.OneClassBaggingMiner10;
import PRFramework.Core.SupervisedClassifiers.EmergingPatterns.Miners.OneClassRandomForestMiner;
import PRFramework.Core.SupervisedClassifiers.EmergingPatterns.Miners.TreeBasedMinerWithoutFiltering;
import PRFramework.Core.SupervisedClassifiers.EmergingPatterns.PatternTests.AlwaysTrue;
import PRFramework.Core.SupervisedClassifiers.EmergingPatterns.SubsetRelation;
import weka.classifiers.RandomizableClassifier;
import weka.core.Instances;
import weka.core.OptionMetadata;

import java.util.Collection;
import java.util.Random;

public class PBC4occ extends RandomizableClassifier {
    //TODO Revert to private
    private PRFramework.Core.SupervisedClassifiers.EmergingPatterns.Classifiers.PBC4occ classifier;
    private boolean filter = false;
    protected RefObject<InstanceModel> model;
    private OneClassRandomForestMiner miner;
    private Instances wekaInstances;
    public PBC4occ() {
        this.miner = new OneClassBaggingMiner10();
        miner.setDecisionTreeBuilder(new DecisionTreeBuilder());
        miner.getDecisionTreeBuilder().setDistributionEvaluator(new Twoing());
    }

    @Override
    public void buildClassifier(Instances data) throws Exception {
        wekaInstances = new Instances(data);
        WekaSerializer wekaSeializer = new WekaSerializer();
        model = new RefObject<>(null);
        Collection<Instance> instances = wekaSeializer.Deserialize(wekaInstances, model);
        classifier = new PRFramework.Core.SupervisedClassifiers.EmergingPatterns.Classifiers.PBC4occ();
        classifier.setPatternsMiner(miner);
        Random randNumGen = new Random(getSeed());
        miner.setRandNumGen(randNumGen);
/*
        DecisionTreeBuilder builder = new DecisionTreeBuilder();
        builder.setDistributionEvaluator(new Twoing());
        builder.setStopCondition(new PureNodeStopCondition());
        builder.setSplitIteratorProvider(new StandardSplitIteratorProvider());
        builder.setPruneResult(false);

        miner.setDecisionTreeBuilder(builder);
        miner.setEPTester(new AlwaysTrue());
        miner.setMinePatternsWhileBuildingTree(false);
        miner.setFilterRelation(SubsetRelation.Equal);
        miner.setTreeCount(treeCount);
        miner.setPercent(percent);
*/
        NominalFeature classFeature =  model.argValue.getClassFeature();
        classifier.Train(model.argValue, instances, classFeature);
        System.out.println("Finished training");
    }

    @Override
    public double[] distributionForInstance(weka.core.Instance instance) throws Exception {
        WekaSerializer wekaSeializer = new WekaSerializer();
        return classifier.Classify(wekaSeializer.Deserialize(wekaInstances, instance, model));
    }

    @OptionMetadata(displayName = "miner",
            description = "The CP miner",
            commandLineParamName = "miner",
            commandLineParamSynopsis = "-miner <string>", displayOrder = 0)
    public OneClassRandomForestMiner getMiner() {
        return miner;
    }

    public void setMiner(OneClassRandomForestMiner miner) {
        this.miner = miner;
    }
    @Override
    public String toString() {
        if (classifier != null){
//
//            int numItems = 0;
//            for (IEmergingPattern pattern : classifier.getEmergingPatterns())
//                numItems += pattern.getItems().size();
//            return classifier.getTopPatterns() + "\nNum patterns: " + classifier.getNumPatterns() +
            return classifier.getTopPatterns() + "\n" + classifier.getNumPatterns();
//                    "\nNum items: " + numItems;
        }
        return "";
    }
    public static void main(String[] args) {
        RandomizableClassifier.runClassifier(new PBC4occ(), args);
    }
}
